"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web_ui_1 = __importDefault(require("@hypernetlabs/web-ui"));
const neverthrow_1 = require("neverthrow");
const rxjs_1 = require("rxjs");
const HypernetIFrameProxy_1 = __importDefault(require("../../implementations/proxy/HypernetIFrameProxy"));
class HypernetWebIntegration {
    constructor(iframeURL, defaultGovernanceChainId, theme, debug) {
        this.iframeURL = "http://localhost:5020";
        this.defaultGovernanceChainId = 1337;
        this.debug = false;
        this.selectedStateChannel = {};
        this.getReadyResolved = false;
        this.getRegistriesReadyResolved = false;
        this.getGovernanceReadyResolved = false;
        this.getPaymentsReadyResolved = false;
        this.paymentsInitializing = false;
        let iframeURLWithSearchParams = new URL(iframeURL || this.iframeURL);
        if (defaultGovernanceChainId != null) {
            iframeURLWithSearchParams.searchParams.append("defaultGovernanceChainId", defaultGovernanceChainId.toString());
        }
        if (debug != null) {
            iframeURLWithSearchParams.searchParams.append("debug", debug.toString());
        }
        this.iframeURL = iframeURLWithSearchParams.toString();
        this.debug = debug || this.debug;
        this.defaultGovernanceChainId =
            defaultGovernanceChainId || this.defaultGovernanceChainId;
        // Create a proxy connection to the iframe
        this.core = new HypernetIFrameProxy_1.default(this._prepareIFrameContainer(), this.iframeURL, "hypernet-core-iframe");
        this.UIData = {
            onSelectedStateChannelChanged: new rxjs_1.Subject(),
            onVotesDelegated: new rxjs_1.Subject(),
            onProviderIdProvided: new rxjs_1.Subject(),
            getSelectedStateChannel: () => this.selectedStateChannel,
        };
        this.UIData.onSelectedStateChannelChanged.subscribe((stateChannel) => {
            this.selectedStateChannel = stateChannel;
        });
        this.UIData.onProviderIdProvided.subscribe(() => {
            // Render payments instuction only if getPaymentsReady is being called
            if (this.paymentsInitializing === true) {
                this.webUIClient.payments.renderPaymentsMetamaskInstructionsWidget({
                    showInModal: true,
                });
            }
        });
        if (window.hypernetWebUIInstance) {
            this.webUIClient = window.hypernetWebUIInstance;
        }
        else {
            this.webUIClient = new web_ui_1.default(this.core, this.UIData, this.iframeURL, this.defaultGovernanceChainId, theme, this.debug);
        }
        this.core.onGatewayIFrameDisplayRequested.subscribe((gatewayUrl) => {
            this.currentGatewayUrl = gatewayUrl;
        });
        this.core.onPrivateCredentialsRequested.subscribe(() => {
            //this.webUIClient.renderPrivateKeysModal();
            this.webUIClient.renderMetamaskWarningModal();
        });
        this.core.onWalletConnectOptionsDisplayRequested.subscribe(() => {
            this.webUIClient.renderWalletConnectWidget({ showInModal: true });
        });
    }
    // wait for the core to be intialized
    getReady() {
        if (this.getReadyResolved === true) {
            return (0, neverthrow_1.okAsync)(this.core);
        }
        return this.core
            .activate()
            .andThen(() => this.core.initialize())
            .map(() => {
            this.getReadyResolved = true;
            window.hypernetCoreInstance = this.core;
            return this.core;
        })
            .mapErr((err) => {
            this.webUIClient.renderWarningAlertModal(`an error occurred during initialization of hypernet core${(err === null || err === void 0 ? void 0 : err.message) ? `: ${err === null || err === void 0 ? void 0 : err.message}` : "."}`);
            return new Error("Something went wrong!");
        });
    }
    getRegistriesReady() {
        if (this.getRegistriesReadyResolved === true) {
            return (0, neverthrow_1.okAsync)(this.core);
        }
        return this.core
            .activate()
            .andThen(() => this.core.registries.initializeRegistries())
            .map(() => {
            this.getRegistriesReadyResolved = true;
            window.hypernetCoreInstance = this.core;
            return this.core;
        })
            .mapErr((err) => {
            this.webUIClient.renderWarningAlertModal(`an error occurred during registries initialization of hypernet core${(err === null || err === void 0 ? void 0 : err.message) ? `: ${err === null || err === void 0 ? void 0 : err.message}` : "."}`);
            return new Error("Something went wrong!");
        });
    }
    getGovernanceReady() {
        if (this.getGovernanceReadyResolved === true) {
            return (0, neverthrow_1.okAsync)(this.core);
        }
        return this.core
            .activate()
            .andThen(() => this.core.governance.initializeGovernance())
            .map(() => {
            this.getGovernanceReadyResolved = true;
            window.hypernetCoreInstance = this.core;
            return this.core;
        })
            .mapErr((err) => {
            this.webUIClient.renderWarningAlertModal(`an error occurred during governance initialization of hypernet core${(err === null || err === void 0 ? void 0 : err.message) ? `: ${err === null || err === void 0 ? void 0 : err.message}` : "."}`);
            return new Error("Something went wrong!");
        });
    }
    getPaymentsReady() {
        this.paymentsInitializing = true;
        if (this.getPaymentsReadyResolved === true) {
            this.paymentsInitializing = false;
            return (0, neverthrow_1.okAsync)(this.core);
        }
        return this.core
            .activate()
            .andThen(() => this.core.payments.initializePayments())
            .map(() => {
            this.getPaymentsReadyResolved = false;
            this.paymentsInitializing = false;
            window.hypernetCoreInstance = this.core;
            return this.core;
        })
            .mapErr((err) => {
            this.webUIClient.renderWarningAlertModal(`an error occurred during payments initialization of hypernet core${(err === null || err === void 0 ? void 0 : err.message) ? `: ${err === null || err === void 0 ? void 0 : err.message}` : "."}`);
            this.paymentsInitializing = false;
            return new Error("Something went wrong!");
        });
    }
    displayGatewayIFrame(gatewayUrl) {
        this.core.payments.displayGatewayIFrame(gatewayUrl);
    }
    closeGatewayIFrame(gatewayUrl) {
        this.core.payments.closeGatewayIFrame(gatewayUrl);
    }
    _prepareIFrameContainer() {
        // Create a container element for the iframe proxy
        const iframeContainer = document.createElement("div");
        iframeContainer.id = "__hypernet-protocol-iframe-container__";
        // Add close modal icon to iframe container
        const closeButton = document.createElement("div");
        closeButton.id = "__hypernet-protocol-iframe-close-icon__";
        closeButton.innerHTML = `
      <img id="__hypernet-protocol-iframe-close-img__" src="https://storage.googleapis.com/hypernetlabs-public-assets/hypernet-protocol/Close-big.png" width="20" />
    `;
        iframeContainer.appendChild(closeButton);
        // Add iframe modal style
        const style = document.createElement("style");
        style.appendChild(document.createTextNode(`
        iframe {
          position: absolute;
          display: none;
          border: none;
          width: 550px;
          height: 60%;
          min-height: 200px;
          background-color: white;
          top: 50%;
          left: 50%;
          box-shadow: 0px 4px 20px #000000;
          border-radius: 4px;
          transform: translate(-50%, -50%);
        }
        #__hypernet-protocol-iframe-container__ {
          position: fixed;
          display: none;
          top: 0;
          left: 0;
          height: 100%;
          width: 100%;
          background-color: rgba(0,0,0,0.6);
          z-index: 999999 !important;
        }
        #__hypernet-protocol-iframe-close-icon__ {
          z-index: 2;
          position: absolute;
          height: 60%;
          top: 50%;
          left: 50%;
          transform: translate(calc(-50% + 263px), -50%);
        }
        #__hypernet-protocol-iframe-close-img__{
          cursor: pointer;
        }
    `));
        document.head.appendChild(style);
        // Attach everything to the body
        document.body.appendChild(iframeContainer);
        const closeImg = document.getElementById("__hypernet-protocol-iframe-close-img__");
        closeImg === null || closeImg === void 0 ? void 0 : closeImg.addEventListener("click", (e) => {
            if (this.currentGatewayUrl != null) {
                this.core.payments.closeGatewayIFrame(this.currentGatewayUrl);
                this.currentGatewayUrl = null;
            }
            iframeContainer.style.display = "none";
        }, false);
        return iframeContainer;
    }
}
exports.default = HypernetWebIntegration;
window.HypernetWebIntegration = HypernetWebIntegration;
//# sourceMappingURL=HypernetWebIntegration.js.map